#! /usr/bin/python
# -*- Python -*-

import sys
sys.path.insert(0, '../pyalsa')
del sys
from alsamemdebug import debuginit, debug, debugdone
import alsamixer

def print_elem(e):
	direction = ["playback", "capture"]

	print "Mixer Element '%s:%i':" % (e.name, e.index)
	print '  is_active: %s' % e.is_active
	print '  is_enumerated: %s' % e.is_enumerated
	print '  has_common_volume: %s' % e.has_common_volume
	print '  has_common_switch: %s' % e.has_common_switch
	print '  has_capture_switch_exclusive: %s' % e.has_capture_switch_exclusive
	if e.has_switch(True):
		print '  getCaptureGroup: %s' % e.get_capture_group
	for capture in [False, True]:
		print '  is_%s_mono: %s' % (direction[capture], e.is_mono(capture))
		print '  has_%s_volume: %s' % (direction[capture], e.has_volume(capture))
		if e.has_volume(capture):
			print '  get_%s_volume_range: %s' % (direction[capture], e.get_volume_range(capture))
			print '  get_%s_volume_range_dB: %s' % (direction[capture], e.get_volume_range_dB(capture))
			print '  get_%s_volume_tuple: %s' % (direction[capture], e.get_volume_tuple(capture)) 
			print '  get_%s_volume_array: %s' % (direction[capture], e.get_volume_array(capture)) 
			vrange = e.get_volume_range(capture)
			if vrange:
				for vol in range(vrange[0], vrange[1]+1):
					print '    vol %i == %idB' % (vol, e.ask_volume_dB(vol, capture))
			dbrange = e.get_volume_range_dB(capture)
			if dbrange:
				for vol in range(dbrange[0], dbrange[1]+1):
					print '    dBvol %i == %i' % (vol, e.ask_dB_volume(vol, -1, capture))
		print '  has_%s_switch: %s' % (direction[capture], e.has_switch(capture))
		if e.has_switch(capture):
			print '  get_%s_switch_tuple: %s' % (direction[capture], e.get_switch_tuple(capture)) 
		for channel in range(alsamixer.channel_id['LAST']+1):
			if e.has_channel(channel, capture):
				print  '  has_%s_channel%s: %s' % (direction[capture], channel, alsamixer.channel_name[channel])

debuginit()

print 'channel_id:'
print alsamixer.channel_id

print 'channel_name:'
print alsamixer.channel_name

print 'regopt_abstracts:'
print alsamixer.regopt_abstract

print 'event_mask:'
print alsamixer.event_mask

print 'event_mask_remove:', alsamixer.event_mask_remove

mixer = alsamixer.Mixer()
mixer.attach()
mixer.load()
print 'Element Count:', mixer.count
print 'Elements:'
print mixer.list()
element = alsamixer.Element(mixer, "PCM")
element.set_volume_array([128, 128])
print_elem(element)
element.set_volume_tuple([127, 127])
print_elem(element)
print_elem(alsamixer.Element(mixer, "Off-hook"))

debug([element])
del element
debug([mixer])
del mixer

debugdone()



#! /usr/bin/python
# Sample code for pyalsa Sequencer binding
# by Aldrin Martoq <amartoq@dcc.uchile.cl>
# version 2008012401 (UTC: 1201153962 secs since the epoch)
#
# This code is in the public domain,
# use it as base for creating your pyalsa
# sequencer application.

import sys
sys.path.insert(0, '../pyalsa')

import alsaseq
import time
from alsamemdebug import debuginit, debug, debugdone

debuginit()

seq = alsaseq.Sequencer()

def dump(event):
    print "event: %s" % event
    print "  ",
    for attr in alsaseq.SeqEvent.__dict__:
        if attr.startswith('is_'):
            t = event.__getattribute__(attr)
            if t:
                print "%s" % attr,
    print
    data = event.get_data()
    print "  data=%s" % data

print "sequencer: %s" % seq

port_id = seq.create_simple_port('hola', alsaseq.SEQ_PORT_TYPE_APPLICATION,
                                 alsaseq.SEQ_PORT_CAP_SUBS_READ | alsaseq.SEQ_PORT_CAP_READ | alsaseq.SEQ_PORT_CAP_WRITE | alsaseq.SEQ_PORT_CAP_SUBS_WRITE
                                 )

print "portid: %d" % port_id

c=-2
wait = 5000

while True:
    if c == -1:
        src = (alsaseq.SEQ_CLIENT_SYSTEM,alsaseq.SEQ_PORT_SYSTEM_ANNOUNCE)
        dest = (seq.client_id, port_id)
        print 'connecting %s -> %s' % (src, dest)
        seq.connect_ports(src, dest)
    if c == 5:
        break
    print 'waiting %s...' % wait
    events = seq.receive_events(wait)
    for event in events:
        c = 0
        dump(event)
        del event
    c += 1

debug([seq])

debugdone()



#! /usr/bin/python

# aseqdump.py -- python port of aplaymidi
# Copyright (C) 2008 Aldrin Martoq <amartoq@dcc.uchile.cl>
#
# Based on code from aseqdump.c, from ALSA project
# Copyright (C) 2005 Clemens Ladisch <clemens@ladisch.de>
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA


import sys
sys.path.insert(0, '../pyalsa')

import getopt
import os
import struct
import alsaseq
import traceback
import time
from alsaseq import *

def errormsg(msg, *args):
    """ prints an error message to stderr """
    sys.stderr.write(msg % args)
    sys.stderr.write('\n')
    traceback.print_exc(file=sys.stderr)


def fatal(msg, *args):
    """ prints an error message to stderr, and dies """
    errormsg(msg, *args)
    sys.exit(1)


def init_seq():
    """ opens an alsa sequencer """
    try:
        sequencer = Sequencer(name = 'default',
                              clientname = 'aseqdump.py',
                              streams = SEQ_OPEN_DUPLEX,
                              mode = SEQ_BLOCK)
        return sequencer
    except SequencerError, e:
        fatal("open sequencer: %e", e)


def parse_ports(sequencer, portspec):
    """ parses one or more port addresses from the string, separated by ','
    example: 14:0,Timidity
    """
    portlist = []
    if portspec == None:
        return portlist
    ports = portspec.split(',')
    for port in ports:
        try:
            client, port = sequencer.parse_address(port)
            portlist.append((client, port))
        except SequencerError, e:
            fatal("Failed to parse port %s - %s", port, e)
    return portlist


def create_source_port(sequencer):
    try :
        port = sequencer.create_simple_port(name = 'aseqdump.py',
                                            type = SEQ_PORT_TYPE_MIDI_GENERIC
                                            | SEQ_PORT_TYPE_APPLICATION,
                                            caps = SEQ_PORT_CAP_WRITE
                                            | SEQ_PORT_CAP_SUBS_WRITE)
        return port
    except SequencerError, e:
        fatal("Failed to create port - %s", e)


def connect_ports(sequencer, port_id, ports):
    client_id = sequencer.client_id
    for client, port in ports:
        sequencer.connect_ports((client, port),(client_id, port_id))

def dump_event(event):
    print "%3d:%-3d" % ((event.source)[0], (event.source)[1]),
    type = event.type
    data = event.get_data()
    if type == SEQ_EVENT_NOTEON:
        print "Note on                %2d %3d %3d" % \
            (data['note.channel'], data['note.note'], data['note.velocity'])
    elif type == SEQ_EVENT_NOTEOFF:
        print "Note off               %2d %3d %3d" % \
            (data['note.channel'], data['note.note'], data['note.velocity'])
    elif type == SEQ_EVENT_KEYPRESS:
        print "Polyphonic aftertouch  %2d %3d %3d" % \
            (data['note.channel'], data['note.note'], data['note.velocity'])
    elif type == SEQ_EVENT_CONTROLLER:
        print "Control change         %2d %3d %3d" % \
            (data['control.channel'], data['control.param'], data['control.value'])
    elif type == SEQ_EVENT_PGMCHANGE:
        print "Program change         %2d %3d" % \
            (data['control.channel'], data['control.value'])
    else:
        print "Event type %d" % type

def list_ports():
    sequencer = init_seq()
    print " Port    Client name                      Port name";
    clientports = sequencer.connection_list()
    for connections in clientports:
        clientname, clientid, connectedports = connections
        for port in connectedports:
            portname, portid, connections = port
            portinfo = sequencer.get_port_info(portid, clientid)
            caps = portinfo['capability']
            if caps & (SEQ_PORT_CAP_READ | SEQ_PORT_CAP_SUBS_READ):
                print "%3d:%-3d  %-32.32s %s" % (clientid, portid, clientname, portname)

def usage():
    print "Usage: %s [options]\n" \
        "\nAvailable options:\n" \
        "-h, --help                  this help\n" \
        "-V, --version               show version\n" \
        "-l, --list                  list input ports\n" \
        "-p, --port=client:port,...  source port(s)\n" \
        % (sys.argv[0])

def version():
    print "aseqdump.py asoundlib version %s" % (SEQ_LIB_VERSION_STR)


def main():
    sequencer = init_seq()
    ports = []
    end_delay = 2

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hVlp:", ["help", "version", "list", "port="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif o in ("-V", "--version"):
            version()
            sys.exit(0)
        elif o in ("-l", "--list"):
            list_ports()
            sys.exit(0)
        elif o in ("-p", "--port"):
            ports = parse_ports(sequencer, a)
        elif o in ("-d", "--delay"):
            end_delay = int(a)

    source_port = create_source_port(sequencer)
    connect_ports(sequencer, source_port, ports)

    sequencer.mode = SEQ_NONBLOCK

    if len(ports) > 0:
        print "Waiting for data.",
    else:
        print "Waiting for data at port %d:0." % sequencer.client_id,

    print "Press Ctrl+C to end."
    print "Source_ Event_________________ Ch _Data__"

    while True:
        try:
            eventlist = sequencer.receive_events(timeout=10069, maxevents = 1)
            for event in eventlist:
                dump_event(event)
        except KeyboardInterrupt:
            pass
            break



if __name__ == '__main__':
    main()



#! /usr/bin/python
# -*- Python -*-

import sys
sys.path.insert(0, '../pyalsa')
del sys
import select
import alsahcontrol

def parse_event_mask(events):
	if events == 0:
		return 'None'
	if events == alsahcontrol.event_mask_remove:
		return 'Removed'
	s = ''
	for i in alsahcontrol.event_mask.keys():
		if events & alsahcontrol.event_mask[i]:
			s += '%s ' % i
	return s[:-1]

def event_callback(element, events):

	info = alsahcontrol.Info(element)
	value = alsahcontrol.Value(element)
	print 'CALLBACK (DEF)! [%s] %s:%i' % (parse_event_mask(events), element.name, element.index)
	print '  ', value.get_tuple(info.type, info.count)


class MyElementEvent:

	def callback(self, element, events):
		info = alsahcontrol.Info(element)
		value = alsahcontrol.Value(element)
		print 'CALLBACK (CLASS)! [%s] %s:%i' % (parse_event_mask(events), element.name, element.index)
		print '  ', value.get_tuple(info.type, info.count)

hctl = alsahcontrol.HControl(mode=alsahcontrol.open_mode['NONBLOCK'])
list = hctl.list()
element1 = alsahcontrol.Element(hctl, list[0][1:])
element1.set_callback(event_callback)
element2 = alsahcontrol.Element(hctl, list[1][1:])
element2.set_callback(MyElementEvent())
nelementid = (alsahcontrol.interface_id['MIXER'], 0, 0, "Z Test Volume", 1)
try:
	hctl.element_remove(nelementid)
except IOError:
	pass
hctl.element_new(alsahcontrol.element_type['INTEGER'],
		nelementid, 2, 0, 100, 1)
hctl.element_unlock(nelementid)
# handleEvents() must be here to update internal alsa-lib's element list
hctl.handle_events()
element3 = alsahcontrol.Element(hctl, nelementid)
element3.set_callback(event_callback)
print 'Watching (DEF): %s,%i' % (element1.name, element1.index)
print 'Watching (CLASS): %s,%i' % (element2.name, element2.index)
print 'Watching (DEF): %s,%i' % (element3.name, element3.index)
poller = select.poll()
hctl.register_poll(poller)
while True:
	poller.poll()
	print 'Poll OK!'
	hctl.handle_events()



#! /usr/bin/python
# -*- Python -*-

import sys
sys.path.insert(0, '../pyalsa')
del sys
import alsacontrol

ctl = alsacontrol.Control()
print 'Card info:', ctl.card_info()
try:
  print 'Hwdep devices:', ctl.hwdep_devices()
except IOError, msg:
  print 'No hwdep devices:', msg
try:
  print 'PCM devices:', ctl.pcm_devices()
except IOError, msg:
  print 'No PCM devices:', msg
try:
  print 'Rawmidi devices:', ctl.rawmidi_devices()
except IOError, msg:
  print 'No rawmidi devices:', msg



#! /usr/bin/python
# -*- Python -*-

import sys
import gc

def debuginit():
	gc.set_debug(gc.DEBUG_LEAK | gc.DEBUG_STATS)
	print "GC DEBUG: init."

def debug(what):
	from sys import getrefcount
	from gc import get_referrers
	for o in what:
		if getrefcount(o) - 3 != 1 or \
		   len(get_referrers(o)) - 2 != 1:
			print 'GD DEBUG LEAK:', o, hex(id(o)), type(o), getrefcount(o)-3, len(get_referrers(o))-2

def debugdone():
	gc.collect()
	print "GC DEBUG LEAK done."
	None



#! /usr/bin/python
# -*- Python -*-

import sys
sys.path.insert(0, '../pyalsa')
del sys
import alsacard

print 'asoundlibVersion:', alsacard.asoundlib_version()
print 'cardLoad:', alsacard.card_load(0)
print 'cardList:', alsacard.card_list()
print 'deviceNameHint for all cards:'
print alsacard.device_name_hint(-1, "pcm")
for card in alsacard.card_list():
	print 'deviceNameHint for card #%i:' % card
	print alsacard.device_name_hint(card, "pcm")



#! /usr/bin/python
# -*- Python -*-

from pyalsa.alsahcontrol import HControl, Element, Info

hctl = HControl(name='hw:1')
list = hctl.list()
for id in list:
	elem = Element(hctl, id[1:])
	info = Info(elem)
	if info.is_user:
		print 'Removing element %s' % repr(id)
		hctl.element_remove(id[1:])



#! /usr/bin/python

# aconnect.py -- python port of aconnect
# Copyright (C) 2008 Aldrin Martoq <amartoq@dcc.uchile.cl>
#
# Based on code from aconnect.c from ALSA project
# Copyright (C) 1999 Takashi Iwai
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA


import sys
sys.path.insert(0, '../pyalsa')

import getopt
import os
from alsaseq import *

LIST_INPUT=1
LIST_OUTPUT=2

def errormsg(msg, *args):
    """ prints an error message to stderr """
    sys.stderr.write(msg % args)
    sys.stderr.write('\n')
    traceback.print_exc(file=sys.stderr)

def fatal(msg, *args):
    """ prints an error message to stderr, and dies """
    errormsg(msg, *args)
    sys.exit(1)

def init_seq():
    """ opens an alsa sequencer """
    try:
        sequencer = Sequencer(name = 'default',
                              clientname = 'aconnect.py',
                              streams = SEQ_OPEN_DUPLEX,
                              mode = SEQ_BLOCK)
        return sequencer
    except SequencerError, e:
        fatal("open sequencer: %e", e)


def usage():
    print  \
        "aconnect - python ALSA sequencer connection manager\n" \
        "Copyright (C) 1999-2000 Takashi Iwai\n" \
        "Copyright (C) 2008 Aldrin Martoq <amartoq@dcc.uchile.cl>\n" \
        "Usage: \n" \
        " * Connection/disconnection between ports\n" \
        "   %s [-options] sender receiver\n" \
        "     sender, receiver = client:port pair\n" \
        "     -d,--disconnect     disconnect\n" \
        "     -e,--exclusive      exclusive connection\n" \
        "     -r,--real #         convert real-time-stamp on queue\n" \
        "     -t,--tick #         convert tick-time-stamp on queue\n" \
        " * List connected ports (no subscription action)\n" \
        "   %s -i|-o [-options]\n" \
        "     -i,--input          list input (readable) ports\n" \
        "     -o,--output         list output (writable) ports\n" \
        "     -l,--list           list current connections of each port\n" \
        " * Remove all exported connections\n" \
        "     -x,--removeall" \
        % (sys.argv[0], sys.argv[0])

def do_list_subs(conn, text):
    l = []
    for c, p, i in conn:
        s = ""
        if i['exclusive']:
            s+= "[ex]"
        if i['time_update']:
            if i['time_real']:
                s += "[%s:%s]" % ("real", i['queue'])
            else:
                s += "[%s:%s]" % ("tick", i['queue'])
        s = "%d:%d%s" % (c, p, s)
        l.append(s)
    if len(l):
        print "\t%s: %s" % (text, ', '.join(l))


def do_list_ports(sequencer, list_perm, list_subs):
    connectionlist = sequencer.connection_list()
    for connections in connectionlist:
        clientname, clientid, connectedports = connections
        clientinfo = sequencer.get_client_info(clientid)
        type = clientinfo['type']
        count = []
        if type == SEQ_USER_CLIENT:
            type = 'user'
        else:
            type = 'kernel'
        for port in connectedports:
            portname, portid, portconns = port
            portinfo = sequencer.get_port_info(portid, clientid)
            caps = portinfo['capability']
            if not (caps & SEQ_PORT_CAP_NO_EXPORT):
                reads = SEQ_PORT_CAP_READ | SEQ_PORT_CAP_SUBS_READ
                write = SEQ_PORT_CAP_WRITE | SEQ_PORT_CAP_SUBS_WRITE
                if (list_perm & LIST_INPUT) and (caps & reads == reads):
                    count.append(port)
                elif (list_perm & LIST_OUTPUT) and (caps & write == write):
                    count.append(port)
        if len(count) > 0:
            print "client %d: '%s' [type=%s]" % (clientid, clientname, type)
            for port in count:
                portname, portid, portconns = port
                print "  %3d '%-16s'" % (portid, portname)
                if list_subs:
                    readconn, writeconn = portconns
                    do_list_subs(readconn, "Connecting To")
                    do_list_subs(writeconn, "Connected From")

def do_unsubscribe(sequencer, args):
    sender = sequencer.parse_address(args[0])
    dest = sequencer.parse_address(args[1])
    try:
        sequencer.get_connect_info(sender, dest)
    except SequencerError:
        print 'No subscription is found'
        return
    try:
        sequencer.disconnect_ports(sender, dest)
    except SequencerError, e:
        fatal("Failed to disconnect ports %s->%s - %s", sender, dest, e)

def do_subscribe(sequencer, args, queue, exclusive, convert_time, convert_real):
    sender = sequencer.parse_address(args[0])
    dest = sequencer.parse_address(args[1])
    try:
        sequencer.get_connect_info(sender, dest)
        print "Connection is already subscribed"
        return
    except SequencerError:
        pass
    sequencer.connect_ports(sender, dest, queue, exclusive, convert_time, convert_real)

def main():
    sequencer = init_seq()
    command = 'subscribe'
    list_perm = 0
    list_subs = 0
    exclusive = 0
    convert_time = 0
    convert_real = 0
    queue = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:], "dior:t:elx", ["disconnect", "input", "output", "real=", "tick=", "exclusive", "list", "removeall"])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for o, a in opts:
        if o in ("-d", "--disconnect"):
            command = 'unsubscribe'
        elif o in ("-i", "--input"):
            command = 'list'
            list_perm |= LIST_INPUT
        elif o in ("-o", "--output"):
            command = 'list'
            list_perm |= LIST_OUTPUT
        elif o in ("-e", "--exclusive"):
            exclusive = 1
        elif o in ("-r", "--real"):
            queue = int(a)
            convert_time = 1
            convert_real = 1
        elif o in ("-t", "--tick"):
            queue = int(a)
            convert_time = 1
            convert_real = 0
        elif o in ("-l", "--list"):
            list_subs = 1
        elif o in ("-x", "--removeall"):
            command = 'removeall'

    if command == 'list':
        do_list_ports(sequencer, list_perm, list_subs)
        return

    if len(args) != 2:
        usage()
        sys.exit(2)

    if command == 'unsubscribe':
        do_unsubscribe(sequencer, args)
    elif command == 'subscribe':
        do_subscribe(sequencer, args, queue, exclusive, convert_time, convert_real)


if __name__ == '__main__':
    main()



#! /usr/bin/python
# Sample code for pyalsa Sequencer binding
# by Aldrin Martoq <amartoq@dcc.uchile.cl>
# version 2008012401 (UTC: 1201153962 secs since the epoch)
#
# This code is in the public domain,
# use it as base for creating your pyalsa
# sequencer application.

import sys
sys.path.insert(0, '../pyalsa')

from alsaseq import *
import time
from alsamemdebug import debuginit, debug, debugdone

debuginit()

def findmidiport():
    for connections in seq.connection_list():
        cname, cid, ports = connections
        # skip midi through
        if cname == 'Midi Through':
            continue
        for port in ports:
            pname, pid, pconns = port
            pinfo = seq.get_port_info(pid, cid)
            type = pinfo['type']
            caps = pinfo['capability']
            if type & SEQ_PORT_TYPE_MIDI_GENERIC and caps & (SEQ_PORT_CAP_WRITE):
                print "Using port: %s:%s" % (cname, pname)
                return (cid, pid)
    print "No midi port found -- install timidity or other software synth for testing!"
    sys.exit(0)

# create sequencer
seq = Sequencer()

# find midi port
cid, pid = findmidiport()

# create a queue
queue = seq.create_queue()
seq.start_queue(queue)
tempo, ppq = seq.queue_tempo(queue)
print "tempo: %d ppq: %d" % (tempo, ppq)

# play notes: DO RE MI FA SOL LA
notes = [0x40, 0x42, 0x44, 0x45, 0x47, 0x49]
event = SeqEvent(SEQ_EVENT_NOTE)
for note in notes:
    event.dest = (cid, pid)
    event.queue = queue
    event.time += ppq
    event.set_data({'note.note' : note, 'note.velocity' : 64, 'note.duration' : ppq , 'note.off_velocity' : 64})
    print 'event: %s %s' % (event, event.get_data())
    seq.output_event(event)
    seq.drain_output()
    seq.sync_output_queue()

# stop queue and delete queue
seq.stop_queue(queue)
seq.delete_queue(queue)

debug([seq])

# close sequencer
del seq


debugdone()



#! /usr/bin/python
# -*- Python -*-

import sys
sys.path.insert(0, '../pyalsa')
del sys
import alsahcontrol

def info(element):
	info = alsahcontrol.Info(element)
	enumerated = alsahcontrol.element_type['ENUMERATED']
	integer = alsahcontrol.element_type['INTEGER']
	integer64 = alsahcontrol.element_type['INTEGER64']
	for a in dir(info):
		if a.startswith('__'):
			continue
		if a in ['items', 'item_names'] and info.type != enumerated:
			continue
		if a in ['min', 'max', 'step'] and info.type != integer:
			continue
		if a in ['min64', 'max64', 'step64'] and info.type != integer64:
			continue
		extra = ''
		if a == 'type':
			extra = ' (%s)' % alsahcontrol.element_type_name[info.type]
		print '  %s: %s%s' % (a, getattr(info, a), extra)

def value(element):
	info = alsahcontrol.Info(element)
	value = alsahcontrol.Value(element)
	for a in dir(value):
		if a.startswith('__'):
			continue
		print '  %s: %s' % (a, getattr(value, a))
	values = value.get_tuple(info.type, info.count)
	print '  Values: ', values
	value.set_tuple(info.type, values)
	value.read()
	if info.is_writable:
		value.write()

print 'interface_id:'
print '  ', alsahcontrol.interface_id
print 'interface_name:'
print '  ', alsahcontrol.interface_name
print 'element_type:'
print '  ', alsahcontrol.element_type
print 'element_type_name:'
print '  ', alsahcontrol.element_type_name
print 'event_class:'
print '  ', alsahcontrol.event_class
print 'event_mask:'
print '  ', alsahcontrol.event_mask
print 'event_mask_remove:', alsahcontrol.event_mask_remove
print '  ', alsahcontrol.open_mode
print 'event_mask_remove:', alsahcontrol.open_mode

hctl = alsahcontrol.HControl()
print 'Count: ', hctl.count
list = hctl.list()
print 'List:'
print list
for l in list:
	print '*****'
	element1 = alsahcontrol.Element(hctl, l[1:])
	info(element1)
	print '-----'
	value(element1)
del hctl



#! /usr/bin/python
# -*- Python -*-

import sys
sys.path.insert(0, '../pyalsa')
del sys
import alsamixer
import select

def parse_event_mask(events):
	if events == 0:
		return 'None'
	if events == alsamixer.event_mask_remove:
		return 'Removed'
	s = ''
	for i in alsamixer.event_mask.keys():
		if events & alsamixer.event_mask[i]:
			s += '%s ' % i
	return s[:-1]

def event_callback(element, events):

	print 'CALLBACK (DEF)! [%s] %s:%i' % (parse_event_mask(events), element.name, element.index)
	print '  ', element.get_volume_tuple(), element.get_switch_tuple()


class MyElementEvent:

	def callback(self, element, events):
		print 'CALLBACK (CLASS)! [%s] %s:%i' % (parse_event_mask(events), element.name, element.index)
		print '  ', element.get_volume_tuple(), element.get_switch_tuple()


mixer = alsamixer.Mixer()
mixer.attach()
mixer.load()

element1 = alsamixer.Element(mixer, "Front")
element1.set_callback(event_callback)
element2 = alsamixer.Element(mixer, "PCM")
element2.set_callback(MyElementEvent())

poller = select.poll()
mixer.register_poll(poller)
while True:
	poller.poll()
	print 'Poll OK!'
	mixer.handle_events()



#! /usr/bin/python
# -*- Python -*-

import os
import sys
try:
	from setuptools import setup, Extension
except ImportError:
	from distutils.core import setup, Extension

VERSION='1.0.26'

if os.path.exists("version"):
	fp = open("version", "r")
	ver = fp.readline()
	fp.close()
	ver = ver[:-1]
else:
	ver = None
if ver != VERSION:
	fp = open("version", "w+")
	fp.write(VERSION + '\n')
	fp.close()
del fp
setup(
	name='pyalsa',
        version=VERSION,
        author="The ALSA Team",
        author_email='alsa-devel@alsa-project.org',
        ext_modules=[
          Extension('pyalsa.alsacard',
		    ['pyalsa/alsacard.c'],
                    include_dirs=[],
                    library_dirs=[],
                    libraries=['asound']),
          Extension('pyalsa.alsacontrol',
		    ['pyalsa/alsacontrol.c'],
                    include_dirs=[],
                    library_dirs=[],
                    libraries=['asound']),
          Extension('pyalsa.alsahcontrol',
		    ['pyalsa/alsahcontrol.c'],
                    include_dirs=[],
                    library_dirs=[],
                    libraries=['asound']),
          Extension('pyalsa.alsamixer',
		    ['pyalsa/alsamixer.c'],
                    include_dirs=[],
                    library_dirs=[],
                    libraries=['asound']),
          Extension('pyalsa.alsaseq',
		    ['pyalsa/alsaseq.c'],
                    include_dirs=[],
                    library_dirs=[],
                    libraries=['asound']),
	],
        packages=['pyalsa'],
        scripts=[]
)

uname = os.uname()
a = 'build/lib.%s-%s-%s' % (uname[0].lower(), uname[4], sys.version[:3])
for f in ['alsacard.so', 'alsacontrol.so', 'alsahcontrol.so', 'alsamixer.so', 'alsaseq.so']:
        if not os.path.exists('pyalsa/%s' % f):
                a = '../build/lib.%s-%s-%s/pyalsa/%s' % (uname[0].lower(),
                        uname[4], sys.version[:3], f)
                print a, f
                os.symlink(a, 'pyalsa/%s' % f)



#! /usr/bin/python
# Sample code for pyalsa Sequencer binding
# by Aldrin Martoq <amartoq@dcc.uchile.cl>
# version 2008012401 (UTC: 1201153962 secs since the epoch)
#
# This code is in the public domain,
# use it as base for creating your pyalsa
# sequencer application.

import sys
sys.path.insert(0, '../pyalsa')
del sys
from alsamemdebug import debuginit, debug, debugdone
import alsaseq


def dump_portinfo(dict):
    def dl(dict, key):
        if dict.has_key(key):
            return "'" + str(dict[key]) + "'"
        return "N/A"
    def da(dict, key, search):
        tmp = None
        if dict.has_key(key):
            for k in search:
                if k & dict[key]:
                    if tmp == None:
                        tmp = str(search[k])
                    else:
                        tmp += " " + str(search[k])
            if tmp == None:
                tmp = "UNKNOWN(%d)" % (dict[key])
        else:
            tmp = "N/A"
        return tmp

    return "name=%s capability=%s type=%s" % (dl(dict, 'name'), da(dict, 'capability', alsaseq._dportcap), da(dict, 'type', alsaseq._dporttype))

def dump_list(connections, simple=True):
    for clientports in connections:
        clientname, clientid, portlist = clientports
        print "    client: %3d    %s" % (clientid, clientname),
        if not simple:
            clientinfo = sequencer.get_client_info(clientid)
            print "\t[%s]" % clientinfo
        else:
            print
        for port in portlist:
            portname, portid, connections = port
            print "      port: %3d:%-2d +-%s" % (clientid, portid, portname),
            if not simple:
                portinfo = sequencer.get_port_info(portid, clientid)
                print "\t[%s]" % (dump_portinfo(portinfo))
            else:
                print
            readconn, writeconn = connections
            for c,p,i in readconn:
                if not simple:
                    print "        connection   to: %d:%d %s" % (c,p, i)
                else:
                    print "        connection   to: %d:%d" % (c,p)
            for c,p,i in writeconn:
                if not simple:
                    print "        connection   to: %d:%d %s" % (c,p, i)
                else:
                    print "        connection   to: %d:%d" % (c,p)


debuginit()

print "01:Creating Sequencer =============================="
sequencer = alsaseq.Sequencer()
# other examples:
# sequencer = alsaseq.Sequencer("hw", "myfancyapplication", alsaseq.SEQ_OPEN_INPUT, alsaseq.SEQ_BLOCK)
# sequencer = alsaseq.Sequencer(clientname='python-test', streams=alsaseq.SEQ_OPEN_OUTPUT)
print "    sequencer:  %s" % sequencer
print "    name:       %s" % sequencer.name
print "    clientname: %s" % sequencer.clientname
print "    streams:    %d (%s)" % (sequencer.streams, str(sequencer.streams))
print "    mode:       %d (%s)" % (sequencer.mode, str(sequencer.mode))
print "    client_id:  %s" % sequencer.client_id
print

print "02:Changing some parameters ========================"
sequencer.clientname = 'pepito'
sequencer.mode = alsaseq.SEQ_BLOCK
print "    clientname: %s" % sequencer.clientname
print "    mode:       %d (%s)" % (sequencer.mode, str(sequencer.mode))
print

print "03:Creating simple port ============================"
port_id = sequencer.create_simple_port("myport", alsaseq.SEQ_PORT_TYPE_APPLICATION,alsaseq.SEQ_PORT_CAP_WRITE | alsaseq.SEQ_PORT_CAP_SUBS_WRITE)
print "    port_id:    %s" % port_id
print

print "04:Getting port info ==============================="
port_info = sequencer.get_port_info(port_id)
print "      -->       %s" % dump_portinfo(port_info)
print "      -->       %s" % port_info
print

print "05:Retrieving clients and connections (as list) ===="
connections = sequencer.connection_list()
print "  %s" % (connections)
print

print "06:Retrieving clients and connections (detailed) ==="
connections = sequencer.connection_list()
dump_list(connections, False)
print

print "07:Connecting 'arbitrary' ports... ================="
source = (alsaseq.SEQ_CLIENT_SYSTEM, alsaseq.SEQ_PORT_SYSTEM_ANNOUNCE)
dest = (sequencer.client_id, port_id)
print "%s ---> %s" % (str(source), str(dest))
sequencer.connect_ports(source, dest)
print

print "08:Retrieving clients and connections (simple) ====="
connections = sequencer.connection_list()
dump_list(connections)
print

print "09:Disconnecting previous 'arbitrary' port ========="
print "%s -X-> %s" % (str(source), str(dest))
sequencer.disconnect_ports(source, dest)
print

print "10:Retrieving clients and connections (simple) ====="
connections = sequencer.connection_list()
dump_list(connections)
print

print "11:Listing known streams constants ================="
print "%s" % alsaseq._dstreams.values()
print

print "12:Listing known mode constants ===================="
print "%s" % alsaseq._dmode.values()
print

print "13:Listing known queue constants ==================="
print "%s" % alsaseq._dqueue.values()
print

print "14:Listing known client type constants ============="
print "%s" % alsaseq._dclienttype.values()
print

print "15:Listing known port caps constants ==============="
print "%s" % alsaseq._dportcap.values()
print

print "16:Listing known port types constants =============="
print "%s" % alsaseq._dporttype.values()
print

print "17:Listing known event type constants =============="
print "%s" % alsaseq._deventtype.values()
print

print "18:Listing known event timestamp constants ========="
print "%s" % alsaseq._deventtimestamp.values()
print

print "19:Listing known event timemode constants =========="
print "%s" % alsaseq._deventtimemode.values()
print

print "20:Listing known client addresses constants ========"
print "%s" % alsaseq._dclient.values()
print

print "21:Listing known port addresses constants =========="
print "%s" % alsaseq._dport.values()
print


print "98:Removing sequencer =============================="
debug([sequencer])
del sequencer
print

print "99:Listing sequencer (using new one) ==============="
dump_list(alsaseq.Sequencer().connection_list())
print

debugdone()
print "seqtest1.py done."



#! /usr/bin/python

# aplaymidi.py -- python port of aplaymidi
# Copyright (C) 2008 Aldrin Martoq <amartoq@dcc.uchile.cl>
#
# Based on code from aplaymidi.c from ALSA project
# Copyright (C) 2004-2006 Clemens Ladisch <clemens@ladisch.de>
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA


import sys
sys.path.insert(0, '../pyalsa')

import getopt
import os
import struct
import alsaseq
import traceback
import time
from alsaseq import *

class SMFError(Exception):
    """ Exception raised for errors while reading a SMF file """
    pass


def errormsg(msg, *args):
    """ prints an error message to stderr """
    sys.stderr.write(msg % args)
    sys.stderr.write('\n')
    traceback.print_exc(file=sys.stderr)


def fatal(msg, *args):
    """ prints an error message to stderr, and dies """
    errormsg(msg, *args)
    sys.exit(1)


def init_seq():
    """ opens an alsa sequencer """
    try:
        sequencer = Sequencer(name = 'default',
                              clientname = 'aplaymidi.py',
                              streams = SEQ_OPEN_DUPLEX,
                              mode = SEQ_BLOCK)
        return sequencer
    except SequencerError, e:
        fatal("open sequencer: %e", e)


def parse_ports(sequencer, portspec):
    """ parses one or more port addresses from the string, separated by ','
    example: 14:0,Timidity
    """
    portlist = []
    if portspec == None:
        return portlist
    ports = portspec.split(',')
    for port in ports:
        try:
            client, port = sequencer.parse_address(port)
            portlist.append((client, port))
        except SequencerError, e:
            fatal("Failed to parse port %s - %s", port, e)
    return portlist


def create_source_port(sequencer):
    try :
        port = sequencer.create_simple_port(name = 'aplaymidi.py',
                                            type = SEQ_PORT_TYPE_MIDI_GENERIC \
                                                | SEQ_PORT_TYPE_APPLICATION,
                                            caps = SEQ_PORT_CAP_NONE)
        return port
    except SequencerError, e:
        fatal("Failed to create port - %s", e)


def create_queue(sequencer):
    try:
        queue = sequencer.create_queue(name = 'aplaymidi')
        return queue
    except SequencerError, e:
        fatal("Failed to create queue - %s", e)


def connect_ports(sequencer, port_id, ports):
    client_id = sequencer.client_id
    for client, port in ports:
        sequencer.connect_ports((client_id, port_id), (client, port))


def read_byte(file):
    """ read a single byte
    raises SMFError if end of file was detected
    """
    s = file.read(1)
    if s == '':
        raise SMFError('End of file reached at position %d' % file.tell())
    return struct.unpack('B', s)[0]


def read_32_le(file):
    """ reads a little-endian 32-bit integer """
    value = 0
    for p in (0, 8, 16, 24):
        c = read_byte(file)
        value |= c << p
    return value


def read_id(file):
    """ reads a 4-char identifier """
    return file.read(4)


def read_int(file, bytes):
    """ reads a fixed-size big-endian number """
    value = 0
    while bytes > 0:
        c = read_byte(file)
        value = (value << 8) | c
        bytes -= 1
    return value


def read_var(file):
    """ reads a variable-length number """
    c = read_byte(file)
    value = c & 0x7f
    if c & 0x80:
        c = read_byte(file)
        value = (value << 7) | (c & 0x7f)
        if c & 0x80:
            c = read_byte(file)
            value = (value << 7) | (c & 0x7f)
            if c & 0x80:
                c = read_byte(file)
                value = (value << 7) | (c & 0x7f)
                if c & 0x80:
                    raise SMFError('Invalid variable-length number at file position %d' % file.tell())
    return value

def skip(file, bytes):
    """ skip the specified number of bytes """
    file.read(bytes)


def read_track(file, chunk_len, ports, smtpe_timing):
    """ reads one complete track from the file """

    tick = 0
    last_cmd = 0
    port = 0
    typemap = {}
    typemap[0x8] = SEQ_EVENT_NOTEOFF
    typemap[0x9] = SEQ_EVENT_NOTEON
    typemap[0xa] = SEQ_EVENT_KEYPRESS
    typemap[0xb] = SEQ_EVENT_CONTROLLER
    typemap[0xc] = SEQ_EVENT_PGMCHANGE
    typemap[0xd] = SEQ_EVENT_CHANPRESS
    typemap[0xe] = SEQ_EVENT_PITCHBEND

    eventlist = []

    track_end = file.tell() + chunk_len
    while file.tell() < track_end:
        delta_ticks = read_var(file)
        if delta_ticks < 0:
            break
        tick += delta_ticks

        c = read_byte(file)

        if c < 0:
            break

        if c & 0x80:
            cmd = c
            if cmd < 0xf0:
                last_cmd = cmd
        else:
            # running status
            cmd = last_cmd
            file.seek(file.tell() - 1)
            if not cmd:
                break


        s = cmd >> 4
        if s >= 0x8 and s <= 0xa:
            event = SeqEvent(type=typemap[s])
            event.dest = ports[port]
            event.time = tick
            event.set_data({'note.channel' : cmd & 0x0f,
                           'note.note' : read_byte(file) & 0x7f,
                           'note.velocity' : read_byte(file) & 0x7f
                           })
            eventlist.append(event)
        elif s == 0xb or s == 0xe:
            event = SeqEvent(type=typemap[s])
            event.dest = ports[port]
            event.time = tick
            event.set_data({'control.channel' : cmd & 0x0f,
                           'control.param' : read_byte(file) & 0x7f,
                           'control.value' : read_byte(file) & 0x7f
                           })
            eventlist.append(event)
        elif s == 0xc or s == 0xd:
            event = SeqEvent(type=typemap[s])
            event.dest = ports[port]
            event.time = tick
            event.set_data({'control.channel' : cmd & 0x0f,
                           'control.value' : read_byte(file) & 0x7f
                           })
            eventlist.append(event)
        elif s == 0xf:
            if cmd == 0xf0 or cmd == 0xf7:
                # sysex
                l = read_var(file)
                if l < 0:
                    break
                if cmd == 0xf0:
                    l += 1
                event = SeqEvent(type=SEQ_EVENT_SYSEX)
                event.dest = ports[port]
                event.time = tick
                sysexdata = []
                if cmd == 0xf0:
                    sysexdata.append(0xf0)
                while len(sysexdata) < l:
                    sysexdata.append(read_byte(file))
                event.set_data({'ext' : sysexdata})
                eventlist.append(event)
            elif cmd == 0xff:
                c = read_byte(file)
                l = read_var(file)
                if l < 0:
                    break
                if c == 0x21:
                    # port number
                    port = read_byte() % len(ports)
                    skip(file, l - 1)
                elif c == 0x2f:
                    # end track
                    track_endtick = tick
                    skip(file, track_end - file.tell())
                    return track_endtick, eventlist
                elif c == 0x51:
                    # tempo
                    if l < 3:
                        break
                    if smtpe_timing:
                        skip(file, l)
                    else:
                        event = SeqEvent(type=SEQ_EVENT_TEMPO)
                        event.dest = (SEQ_CLIENT_SYSTEM, SEQ_PORT_SYSTEM_TIMER)
                        event.time = tick
                        tempo = read_byte(file) << 16
                        tempo |= read_byte(file) << 8
                        tempo |= read_byte(file)
                        event.set_data({'queue.param.value' : tempo})
                        eventlist.append(event)
                        skip(file, l - 3)
                else:
                    # ignore all other meta events
                    skip(file, l)
            else:
                break
        else:
            break

    raise SMFError('Invalid MIDI data at file position (%d)' % file.tell())

def read_smf(file, ports):
    """ reads an entire MIDI file """

    tempo = 0
    ppq = 0
    tracks = []


    header_len = read_int(file, 4)
    if header_len < 6:
        raise SMFError('Invalid header_len: %d' % header_len)

    type = read_int(file, 2)
    if type != 0 and type != 1:
        raise SMFError('Not supported type: %d' % type)

    num_tracks = read_int(file, 2)
    if num_tracks < 1 or num_tracks > 1000:
        raise SMFError('Invalid number of tracks: %d' % num_tracks)

    time_division = read_int(file, 2)
    if time_division < 0:
        raise SMFError('Invalid time division: %d' % time_division)

    smtpe_timing = time_division & 0x8000

    if not smtpe_timing:
        # time division is ticks per quarter
        tempo = 500000
        ppq = time_division
    else:
        i = 0x80 - ((time_division >>8) & 0x7f)
        time_division &= 0xff
        if i == 24:
            tempo = 500000
            ppq = 12 * time_division
        elif i == 25:
            tempo = 400000
            ppq = 10 * time_division
        elif i == 29:
            tempo = 100000000
            ppq = 2997 * time_division
        elif i == 30:
            tempo = 500000
            ppq = 15 * time_division
        else:
            raise SMFError('Invalid number of SMPTE frames per second (%d)'
                           % i)

    # read tracks
    while num_tracks > 0:
        # search for MTrk chunk
        while True:
            id = read_id(file)
            chunk_len = read_int(file, 4)
            if chunk_len < 0 or chunk_len >= 0x10000000:
                raise SMFError('Invalid chunk length %d' % chunk_len)
            if id == 'MTrk':
                break
            skip(file, chunk_len)
        trackevents = read_track(file, chunk_len, ports, smtpe_timing)
        tracks.append(trackevents)
        num_tracks -= 1

    return tempo, ppq, tracks

def read_riff(file, ports):
    """ reads an entire RIFF file """

    raise NotImplementedError("Sorry, I couldn't find any RIFF midi file to implement this!  -- Aldrin Martoq")


def play(sequencer, filename, end_delay, source_port, queue, ports):
    file = sys.stdin
    ok = None
    if filename != '-':
        try:
            file = open(filename, 'rb')
        except IOError, e:
            fatal("Cannot open %s - %s", filename, e)

    id = read_id(file)
    try:
        if id == 'MThd':
            ok = read_smf(file, ports)
        elif id == 'RIFF':
            ok = read_riff(file, ports)
        else:
            raise SMFError('unrecognized id')
    except SMFError, e:
        errormsg("%s is not a Standard MIDI File - %s", filename, e)

    # now play!
    tempo, ppq, tracks = ok
    sequencer.queue_tempo(queue, tempo, ppq)
    sequencer.start_queue(queue)

    # add all events from tracks and sort them based on track, then on time
    itrack = 0
    allevents = []
    for track in tracks:
        endtick, eventlist = track
        for event in eventlist:
            d = {'itrack' : itrack, 'event' : event, 'time' : event.time}
            allevents.append(d)
        itrack += 1
    def compare(x, y):
        ret = x['time'] - y['time']
        if ret == 0:
            ret = x['itrack'] - y['itrack']
        return ret
    allevents.sort(compare)
    max_tick = allevents[-1]['time']

    for d in allevents:
        event = d['event']
        event.source = (0, 0)
        event.queue = queue
        if event.type == SEQ_EVENT_TEMPO:
            event.set_data({'queue.queue' : queue})
        sequencer.output_event(event)

    # schedule queue stop at end of song
    event = SeqEvent(SEQ_EVENT_STOP)
    event.source = (0, 0)
    event.queue = queue
    event.time = max_tick
    event.dest = (SEQ_CLIENT_SYSTEM, SEQ_PORT_SYSTEM_TIMER)
    event.set_data({'queue.queue' : queue})
    sequencer.output_event(event)

    # make sure that the sequencer sees all our events
    sequencer.drain_output()
    sequencer.sync_output_queue()

    time.sleep(end_delay)

    if file != sys.stdin:
        file.close()



def list_ports():
    sequencer = init_seq()
    print " Port    Client name                      Port name";
    clientports = sequencer.connection_list()
    for connections in clientports:
        clientname, clientid, connectedports = connections
        for port in connectedports:
            portname, portid, connections = port
            portinfo = sequencer.get_port_info(portid, clientid)
            type = portinfo['type']
            caps = portinfo['capability']
            if type & SEQ_PORT_TYPE_MIDI_GENERIC and \
                    caps & (SEQ_PORT_CAP_WRITE | SEQ_PORT_CAP_SUBS_WRITE):
                print "%3d:%-3d  %-32.32s %s" % (clientid, portid, clientname, portname)

def usage():
    print "Usage: %s -p client:port[,...] [-d delay] midifile ...\n" \
        "-h, --help                  this help\n" \
        "-V, --version               print current version\n" \
        "-l, --list                  list all possible output ports\n" \
        "-p, --port=client:port,...  set port(s) to play to\n" \
        "-d, --delay=seconds         delay after song ends\n" % (sys.argv[0])

def version():
    print "aplaymidi.py asoundlib version %s" % (SEQ_LIB_VERSION_STR)


def main():
    sequencer = init_seq()
    ports = []
    end_delay = 2

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hVlp:d:", ["help", "version", "list", "port=", "delay="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif o in ("-V", "--version"):
            version()
            sys.exit(0)
        elif o in ("-l", "--list"):
            list_ports()
            sys.exit(0)
        elif o in ("-p", "--port"):
            ports = parse_ports(sequencer, a)
        elif o in ("-d", "--delay"):
            end_delay = int(a)

    if len(ports) < 1:
        ports = parse_ports(sequencer, os.getenv('ALSA_OUTPUT_PORTS'))
        if len(ports) < 1:
            fatal("Please specify at least one port with --port.")

    if len(args) < 1:
        fatal("Please specify a file to play.")

    source_port = create_source_port(sequencer)
    queue = create_queue(sequencer)
    connect_ports(sequencer, source_port, ports)

    for filename in args:
        play(sequencer, filename, end_delay, source_port, queue, ports)


if __name__ == '__main__':
    main()
